@@ -1,6 +1,7 @@
 from abc import abstractmethod
 from asyncio.tasks import Task
 from configparser import ConfigParser
+from logging import getLogger, DEBUG, INFO, WARNING, ERROR
 import threading
 import asyncio
 import queue
@@ -24,7 +25,7 @@ from .unique_brain.constants import *
 from .packet import *
 from .transport import Transport
 
-from .util import now, log, logv
+from .util import now, short_id
 from .pack import PackTuple, PackType, jdpack, jdunpack
 
 EV_CHANGE = "change"
@@ -154,11 +155,11 @@ class EventEmitter:
     def _add_log_report(self, priority: int, text: str, *args: object):
         prefix = self._log_report_prefix()
         msg = prefix + text
-        logger = self.bus.logger
-        if logger:
-            logger.report(priority, msg, *args)
+        logger_server = self.bus.logger_server
+        if not logger_server is None:
+            logger_server.report(priority, msg, *args)
 
-    def log(self, text: str, *args: object):
+    def info(self, text: str, *args: object):
         self._add_log_report(LoggerPriority.LOG, text, *args)
 
     def warn(self, text: str, *args: object):
@@ -180,18 +181,23 @@ def _service_matches(dev: 'Device', serv: bytearray):
             return False
     return True
 
+
 def rand_u64():
     return bytearray([getrandbits(8) for _ in range(8)])
 
+
 def is_raspberrypi():
     # https://raspberrypi.stackexchange.com/questions/5100/detect-that-a-python-program-is-running-on-the-pi
     try:
-        import io
-        with io.open('/sys/firmware/devicetree/base/model', 'r') as m:
-            if 'raspberry pi' in m.read().lower(): return True
-    except Exception: pass
+        from io import open
+        with open('/sys/firmware/devicetree/base/model', 'r') as m:
+            if 'raspberry pi' in m.read().lower():
+                return True
+    except Exception:
+        pass
     return False
 
+
 class Bus(EventEmitter):
     """A Jacdac bus that managed devices, service client, registers."""
 
@@ -210,7 +216,8 @@ class Bus(EventEmitter):
                  hf2_portname: Optional[str] = None,
                  transport_cmd: Optional[str] = None,
                  default_logger_min_priority: Optional[int] = None,
-                 storage_dir: Optional[str] = None
+                 storage_dir: Optional[str] = None,
+                 logger_name: Optional[str] = None
                  ) -> None:
         """Creates a new Jacdac bus.
 
@@ -231,6 +238,7 @@ class Bus(EventEmitter):
             hf2_portname (str, optional): port name exposing HF2 packets.
             transport_cmd (str, optional): name of executable to run as a transport.
             spi (bool, optional): use SPI for transport. Enabled by default if Raspberry Pi and spi is None.
+            logger_name (str, optional): custom logger name.
         """
         super().__init__(self)
 
@@ -238,19 +246,22 @@ class Bus(EventEmitter):
         self.unattached_clients: List['Client'] = []
         self.all_clients: List['Client'] = []
         self.servers: List['Server'] = []
-        self.logger: Optional[LoggerServer] = None
+        self.logger_server: Optional[LoggerServer] = None
         self.role_manager: Optional[RoleManagerServer] = None
         self.pipes: List['InPipe'] = []
         self._event_counter = 0
 
         # merge .ctor configuration with files
         config = ConfigParser()
-        config.read(["./jacdac.ini", os.path.expanduser("~") + "/.jacdac/config.ini", "./setup.cfg"])
+        config.read(["./jacdac.ini", os.path.expanduser("~") +
+                    "/.jacdac/config.ini", "./setup.cfg"])
         if not config.has_section("jacdac"):
             cfg = config.add_section("jacdac")
         cfg = config["jacdac"]
         device_id = device_id or cfg.get(
             "device_id", rand_u64().hex())
+        logger_name = logger_name or cfg.get("logger_name", "jacdac.{}".format(short_id(device_id)))
+        self.logger = getLogger(logger_name)
         self.product_identifier: Optional[int] = product_identifier or cfg.getint(
             "product_identifier", None)
         self.firmware_version: Optional[str] = firmware_version or cfg.get(
@@ -288,7 +299,7 @@ class Bus(EventEmitter):
             self.transports.append(ExecTransport(self.transport_cmd))
         if self.hf2_portname:
             from .transports.hf2 import HF2Transport
-            self.transports.append(HF2Transport(self.hf2_portname))        
+            self.transports.append(HF2Transport(self.hf2_portname))
         if self.spi:
             from .transports.spi import SpiTransport
             self.transports.append(SpiTransport())
@@ -311,7 +322,7 @@ class Bus(EventEmitter):
 
         self.process_thread.start()
 
-        print("starting jacdac, self device {}".format(self.self_device))
+        self.logger.info("starting jacdac, self device {}".format(self.self_device))
 
     def run(self, cb: Callable[..., None], *args: Any):
         if self.process_thread is threading.current_thread():
@@ -336,7 +347,7 @@ class Bus(EventEmitter):
         ctrls = ControlServer(self)  # attach control server
 
         if not self.disable_logger:
-            self.logger = LoggerServer(self)
+            self.logger_server = LoggerServer(self)
 
         if self.storage_dir and not self.disable_role_manager:
             self.role_manager = RoleManagerServer(self)
@@ -390,10 +401,10 @@ class Bus(EventEmitter):
             while ptr < 12 + frame[2]:
                 sz = frame[ptr] + 4
                 pktbytes = frame[0:12] + frame[ptr:ptr+sz]
-                # log("PKT: {}-{} / {}", ptr, len(frame), pktbytes.hex())
+                # self.logger.debug("PKT: {}-{} / {}", ptr, len(frame), pktbytes.hex())
                 pkt = JDPacket(frombytes=pktbytes, sender=sender)
                 if ptr > 12:
-                    pkt.requires_ack = False # only ack once
+                    pkt.requires_ack = False  # only ack once
                 self.process_packet(pkt)
                 # dispatch to other transports
                 self._queue_core(pkt)
@@ -504,7 +515,7 @@ class Bus(EventEmitter):
                         break
 
     def process_packet(self, pkt: JDPacket):
-        logv("route: {}", pkt)
+        # self.debug("route: {}", pkt)
         dev_id = pkt.device_id
         multi_command_class = pkt.multicommand_class
         service_index = pkt.service_index
@@ -540,7 +551,7 @@ class Bus(EventEmitter):
         elif dev_id == self.self_device.device_id and pkt.is_command:
             h = self.servers[pkt.service_index]
             if h:
-                # log(`handle pkt at ${h.name} cmd=${pkt.service_command}`)
+                # self.logger.debug(`handle pkt at ${h.name} cmd=${pkt.service_command}`)
                 h.handle_packet_outer(pkt)
         else:
             if pkt.is_command:
@@ -762,12 +773,12 @@ class RawRegisterClient(EventEmitter):
         else:
             return values[0]
 
-    def bool_value(self, default_value: bool = None) -> Optional[bool]:
+    def bool_value(self, default_value: Optional[bool] = None) -> Optional[bool]:
         """Extracts the value of the first field as a boolean."""
         value = self.value()
         return bool(value) if not value is None else default_value
 
-    def float_value(self, default_value: float = None, scale: int = 1) -> Optional[float]:
+    def float_value(self, default_value: Optional[float] = None, scale: int = 1) -> Optional[float]:
         value = self.value()
         return float(value) * scale if not value is None else default_value
 
@@ -1011,7 +1022,7 @@ class ControlServer(Server):
         self.auto_bind_cnt = 0
 
     def queue_announce(self):
-        logv("announce: %d " % self.restart_counter)
+        # self.debug("announce: %d " % self.restart_counter)
         self.restart_counter += 1
         ids = [s.service_class for s in self.bus. servers]
         rest = self.restart_counter
@@ -1108,8 +1119,21 @@ class LoggerServer(Server):
         return super().handle_packet(pkt)
 
     def report(self, priority: int, msg: str, *args: object):
+        if priority < LoggerPriority.SILENT:
+            severity = DEBUG
+            if priority == LoggerPriority.WARNING:
+                severity = WARNING
+            elif priority == LoggerPriority.ERROR:
+                severity = ERROR
+            elif priority == LoggerPriority.LOG:
+                severity = INFO
+            if len(args) > 0:
+                self.bus.logger.log(severity, msg.format(*args))
+            else:
+                self.bus.logger.log(severity, msg)
+
         if priority >= self.min_priority:
-            log(msg, *args)
+            return
         cmd: int = -1
         if priority == LoggerPriority.DEBUG:
             cmd = JD_LOGGER_CMD_DEBUG
@@ -1117,7 +1141,7 @@ class LoggerServer(Server):
             cmd = JD_LOGGER_CMD_LOG
         elif priority == LoggerPriority.WARNING:
             cmd = JD_LOGGER_CMD_WARN
-        elif priority == JD_LOGGER_CMD_ERROR:
+        elif priority == JD_LOGGER_CMD_ERROR:            
             cmd = JD_LOGGER_CMD_ERROR
         else:
             return
@@ -1694,7 +1718,7 @@ class Device(EventEmitter):
 
     @ property
     def short_id(self):
-        return util.short_id(self.device_id)
+        return short_id(self.device_id)
 
     def __str__(self) -> str:
         return self.short_id
@@ -1781,10 +1805,11 @@ class Device(EventEmitter):
         for c in self.clients:
             if (c.broadcast and c.service_class == service_class) or \
                (not c.broadcast and c.service_index == pkt.service_index):
-                # log(`handle pkt at ${client.role} rep=${pkt.serviceCommand}`)
+                # info(`handle pkt at ${client.role} rep=${pkt.serviceCommand}`)
                 c.device = self
                 c.handle_packet_outer(pkt)
 
+
 class BufferClient(Client):
     _value: bytearray
     _dirty: bool
@@ -1792,12 +1817,13 @@ class BufferClient(Client):
     """
     A client that handles a double-buffer bytes buffer
     """
+
     def __init__(self, bus: Bus, service_class: int, pack_formats: Dict[int, str], role: str) -> None:
         super().__init__(bus, service_class, pack_formats, role)
 
         self._value = bytearray(0)
         self._dirty = False
-    
+
     @property
     def value(self) -> bytearray:
         """
@@ -1816,10 +1842,10 @@ class BufferClient(Client):
     @property
     def dirty(self) -> bool:
         return self._dirty
-    
+
     def set_dirty(self) -> None:
         self._dirty = True
-    
+
     def refresh_value(self) -> None:
         if self._dirty:
             self.register(JD_REG_VALUE).set_values(self._value)
@@ -1833,7 +1859,5 @@ class BufferClient(Client):
                 self._value = self._value + bytearray(length - l)
                 self._dirty = True
             else:
-                self._value = self._value[0:length -1]
+                self._value = self._value[0:length - 1]
                 self._dirty = True
-
-
